# Generated by Django 3.1.5 on 2021-03-15 13:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ingress', '0008_auto_20210315_1454'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='FailedIngressQueue',
            new_name='FailedMessage',
        ),
        migrations.RenameModel(
            old_name='IngressQueue',
            new_name='Message',
        ),
    ]



# Generated by Django 3.1.5 on 2021-03-15 13:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ingress', '0007_auto_20210315_1451'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Endpoint',
            new_name='Collection',
        ),
        migrations.RenameField(
            model_name='collection',
            old_name='url_key',
            new_name='name',
        ),
        migrations.RenameField(
            model_name='failedingressqueue',
            old_name='parse_fail_info',
            new_name='consume_fail_info',
        ),
        migrations.RenameField(
            model_name='failedingressqueue',
            old_name='parse_failed_at',
            new_name='consume_failed_at',
        ),
        migrations.RenameField(
            model_name='failedingressqueue',
            old_name='parse_started',
            new_name='consume_started_at',
        ),
        migrations.RenameField(
            model_name='failedingressqueue',
            old_name='parse_succeeded',
            new_name='consume_succeeded_at',
        ),
        migrations.RenameField(
            model_name='ingressqueue',
            old_name='parse_started',
            new_name='consume_started_at',
        ),
        migrations.RenameField(
            model_name='ingressqueue',
            old_name='parse_succeeded',
            new_name='consume_succeeded_at',
        ),
    ]



from django.apps import AppConfig
from django.core.exceptions import ImproperlyConfigured

from ingress.settings import app_settings


class IngressConfig(AppConfig):
    name = "ingress"

    def ready(self):
        if not (app_settings.PERMISSION_CLASSES or app_settings.AUTHENTICATION_CLASSES):
            raise ImproperlyConfigured(
                "At lease one of INGRESS_PERMISSION_CLASSES and "
                "INGRESS_AUTHENTICATION_CLASSES settings must be set"
            )

        super().ready()



import logging

from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from ingress.models import Collection, Message
from ingress.settings import app_settings

logger = logging.getLogger(__name__)


@method_decorator(csrf_exempt, name="dispatch")
class IngressView(APIView):
    def get_permissions(self):
        """
        Instantiates and returns the list of permissions that this view requires.
        """
        return [permission() for permission in app_settings.PERMISSION_CLASSES]

    def get_authenticators(self):
        """
        Instantiates and returns the list of authenticators that this view can use.
        """
        return [auth() for auth in app_settings.AUTHENTICATION_CLASSES]

    def post(self, request, collection_name):
        # check if the collection exists and is active
        collection = get_object_or_404(Collection, name=collection_name)
        if not collection.ingress_enabled:
            return Response(
                status=status.HTTP_404_NOT_FOUND,
                data="Ingress for collection is disabled",
            )

        raw_data = request.body.decode(app_settings.ENCODING)
        try:
            Message.objects.create(collection=collection, raw_data=raw_data)
            return Response(status=status.HTTP_200_OK)
        except Exception:
            logger.exception("Failed to create Message")
            return Response(status=status.HTTP_500_INTERNAL_SERVER_ERROR)



import logging
from abc import ABC, abstractmethod
from time import sleep

from django.db import transaction

from ingress.models import Collection, Message

log = logging.getLogger(__name__)


class BaseConsumer(ABC):
    """
    Whether or not to immediately remove messages once consumption succeeds.
    If set to False, message.consume_succeeded_at will be set.
    """

    remove_message_on_consumed = True

    """
    Whether or not to set Message.consume_started_at immediately once consumption starts
    """
    set_consume_started_at = False

    @property
    @abstractmethod
    def collection_name(self):
        pass

    @abstractmethod
    def consume_raw_data(self, raw_data):
        """
        Implement consumption of one raw message.
        If it fails an exception must be raised.
        """
        pass

    def on_consume_start(self, message):
        if self.set_consume_started_at:
            message.set_consume_started(save_immediately=True)

    def on_consume_success(self, message):
        if self.remove_message_on_consumed:
            message.delete()
        else:
            message.set_consume_succeeded(save_immediately=True)

    def on_consume_error(self, message):
        message.move_to_failed_queue()

    def consume_message(self, message):
        if message.consume_started:
            return

        self.on_consume_start(message)

        try:
            # A try/except within an atomic transaction is not possible
            # For this reason we add another transaction within this try/except
            # https://docs.djangoproject.com/en/3.1/topics/db/transactions/#controlling-transactions-explicitly
            with transaction.atomic():
                self.consume_raw_data(message.raw_data)
                self.on_consume_success(message)

        except Exception:
            self.on_consume_error(message)

    def consume_iterator(self, message_iterator):
        for message in message_iterator:
            self.consume_message(message)

    def get_default_batch_size(self):
        return 100

    # flake8: noqa: C901
    def consume(
        self, end_at_empty_queue=False, end_at_disabled_consumer=False, batch_size=None
    ):
        if not batch_size:
            batch_size = self.get_default_batch_size()

        try:
            collection = Collection.objects.get(name=self.collection_name)
        except Collection.DoesNotExist:
            log.error(
                f"\n\tNo collection exists with the name '{self.collection_name}'."
                "\n\tDid you forget to create it? Run the command below to create it:"
                f"\n\tpython manage.py add_collection {self.collection_name}\n"
            )
            return

        while True:
            collection.refresh_from_db()
            if not collection.consumer_enabled:
                if end_at_disabled_consumer:
                    break  # For testing purposes
                sleep(10)
                continue

            with transaction.atomic():
                # This locks N records and iterates over them.
                # Parallel workers simply lock the next N records
                # Quote from https://www.postgresql.org/docs/11/sql-select.html :
                # "If a LIMIT is used, locking stops once enough rows have been
                # returned to satisfy the limit"
                messages = (
                    Message.objects.for_collection(collection)
                    .not_consumed()
                    .order_by("created_at")
                    .select_for_update(skip_locked=True)[:batch_size]
                )

                num_messages = messages.count()
                if num_messages > 0:
                    self.consume_iterator(messages.iterator())

                if num_messages < batch_size:
                    if end_at_empty_queue:
                        break  # For testing purposes
                    sleep(1)



from django.core.management.base import BaseCommand

from ingress.models import Collection


class Command(BaseCommand):
    help = "Adds a new collection"

    def add_arguments(self, parser):
        parser.add_argument(
            "name",
            help=(
                "The collection name, used when posting to our api. "
                "For example: with the name being 'people_data_v2', the url will be "
                "'/ingress/people_data_v2' (with no slash at the end)."
            ),
        )

    def handle(self, *args, **options):
        name = options["name"]

        c = Collection(name=name)
        c.full_clean()
        c.save()
        self.stdout.write(
            f"Created Collection '{c.name}' (active: {c.ingress_enabled}, "
            f"consumer enabled: {c.consumer_enabled})"
        )



# Generated by Django 2.2.16 on 2020-11-05 14:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ingress", "0002_auto_20201104_1740"),
    ]

    operations = [
        migrations.CreateModel(
            name="FailedIngressQueue",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("raw_data", models.TextField()),
                ("parse_started", models.DateTimeField(null=True)),
                ("parse_succeeded", models.DateTimeField(null=True)),
                ("parse_failed", models.DateTimeField(null=True)),
                ("parse_fail_info", models.TextField(null=True)),
                (
                    "endpoint",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ingress.Endpoint",
                    ),
                ),
            ],
        ),
    ]



from django.core.management.base import BaseCommand

from ingress.models import Collection


class Command(BaseCommand):
    help = "Remove an existing collection"

    def add_arguments(self, parser):
        parser.add_argument(
            "name",
            help="The collection name, used when posting to our api.",
        )

    def handle(self, *args, **options):
        name = options["name"]

        num_deleted, _ = Collection.objects.filter(name=name).delete()
        if num_deleted > 0:
            self.stdout.write(f"Deleted Collection '{name}'")
        else:
            self.stderr.write(
                f"Did not delete Collection '{name}'. It probably does not exist."
            )



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from rest_framework.settings import perform_import

DEFAULTS = {
    # Whether or not to accept data posted to a non-existing collection.
    "ACCEPT_NEW_COLLECTIONS": False,
    # A list of classpaths to implementations of ingress.consumer.IngressConsumer
    # to handle the data in the queue.
    "CONSUMER_CLASSES": [],
    # A list of authentication classes used in the ingress view.
    # See https://www.django-rest-framework.org/api-guide/authentication/
    "AUTHENTICATION_CLASSES": [],
    # A list of permission classes used in the ingress view.
    # See https://www.django-rest-framework.org/api-guide/permissions/
    "PERMISSION_CLASSES": [],
    # Encoding that the data will be in when posted to the ingress
    "ENCODING": "utf-8",
}

# Which settings to automatically import upon use
IMPORT_STRINGS = ["PERMISSION_CLASSES", "AUTHENTICATION_CLASSES", "CONSUMER_CLASSES"]


class AppSettings:
    def __getattr__(self, attr):
        if attr not in DEFAULTS:
            raise AttributeError(f"Invalid API setting: {attr}")

        # get the setting from the django conf settings (application level)
        try:
            value = getattr(settings, f"INGRESS_{attr}")
        except AttributeError:
            # or obtain a default
            value = DEFAULTS[attr]

        if attr in IMPORT_STRINGS:
            return perform_import(value, attr)
        return value

    def get_consumer_for_collection(self, collection_name):
        consumers = self.CONSUMER_CLASSES or []
        for consumer in consumers:
            if consumer.collection_name == collection_name:
                return consumer

        raise ObjectDoesNotExist(
            f"No consumer exists for collection '{collection_name}'. "
            f"Define it in INGRESS_CONSUMER_CLASSES"
        )


app_settings = AppSettings()



from django.db import models


class MessageQuerySet(models.QuerySet):
    def for_collection(self, collection):
        return self.filter(collection=collection)

    def not_consumed(self):
        return self.filter(consume_started_at__isnull=True)

    def consumed(self):
        return self.filter(consume_succeeded_at__isnull=False)


class FailedMessageQuerySet(MessageQuerySet):
    pass



# Generated by Django 2.2.16 on 2020-11-04 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="IngressQueue",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("endpoint", models.TextField()),
                ("raw_data", models.TextField()),
                ("parse_started", models.DateTimeField(null=True)),
                ("parse_succeeded", models.DateTimeField(null=True)),
                ("parse_failed", models.DateTimeField(null=True)),
                ("parse_fail_info", models.TextField(null=True)),
            ],
        ),
    ]



# from django.core.management.base import BaseCommand
#
# from ingress.models import Endpoint, FailedIngressQueue
#
#
# class ShowFailedMessagesError(Exception):
#     pass
#
#
# class Command(BaseCommand):
#     help = "Show all info for the first failed record. Optionally an url_key can be defined to specify the endpoint."
#
#     def add_arguments(self, parser):
#         parser.add_argument(
#             "url_key",
#             nargs="?",
#             default=None,
#             help="The url_key for the endpoint. Use the 'list_endpoints' command to show the url_keys for "
#             "the existing endpoints.",
#         )
#
#     def handle(self, *args, **options):
#         url_key = options["url_key"]
#         if url_key:
#             try:
#                 endpoint = Endpoint.objects.get(url_key=url_key)
#             except Endpoint.DoesNotExist:
#                 raise ShowFailedMessagesError(
#                     f"\n\nThe endpoint with url_key '{url_key}' does not exist. Nothing has been done.\n\n"
#                 )
#
#             failed_ingresses = FailedIngressQueue.objects.filter(
#                 endpoint=endpoint
#             ).order_by("created_at")
#         else:
#             failed_ingresses = FailedIngressQueue.objects.order_by("created_at")
#
#         if failed_ingresses.count() == 0:
#             raise ShowFailedMessagesError(
#                 "No failed messages were found, nothing to show."
#             )
#
#         table_spacing = "{:<20} {:<50}"
#
#         queue_attrs = (
#             "created_at",
#             "parse_started",
#             "parse_succeeded",
#             "parse_failed",
#             "parse_fail_info",
#             "raw_data",
#         )
#
#         failed_ingress = failed_ingresses.first()
#         self.stdout.write(
#             "\n\n" + table_spacing.format("endpoint", failed_ingress.endpoint.url_key)
#         )
#         for attr in queue_attrs:
#             self.stdout.write(
#                 table_spacing.format(attr, str(getattr(failed_ingress, attr)) or "None")
#             )



from django.core.management.base import BaseCommand

from ingress.models import Message


class Command(BaseCommand):
    def handle(self, *args, **options):
        result = Message.objects.consumed().delete()
        self.stdout.write(f"Deleted {result[0]} consumed messages from the queue.")



import sys
import traceback

from django.db import models
from django.urls import reverse
from django.urls.exceptions import NoReverseMatch
from django.utils import timezone

from ingress.querysets import FailedMessageQuerySet, MessageQuerySet


class Collection(models.Model):
    name = models.SlugField(
        max_length=255,
        unique=True,
        allow_unicode=False,
        help_text=(
            "The end of the url and also the key with which it can be retrieved from "
            "the queue. For example, in the url /ingress/example, the string 'example' "
            "is the name of the collection."
        ),
    )
    ingress_enabled = models.BooleanField(default=True)
    consumer_enabled = models.BooleanField(default=False)

    @property
    def url_path(self):
        try:
            return reverse('ingress', kwargs={'collection_name': self.name})
        except NoReverseMatch:
            return None


class BaseMessage(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)
    updated_at = models.DateTimeField(auto_now=True)
    collection = models.ForeignKey(Collection, on_delete=models.PROTECT, db_index=True)
    raw_data = models.TextField(
        help_text="To store raw received data to be consumed by a separate consumer"
    )
    consume_started_at = models.DateTimeField(null=True, db_index=True)
    consume_succeeded_at = models.DateTimeField(null=True, db_index=True)

    @property
    def consume_started(self):
        return self.consume_started_at is not None

    @property
    def consume_succeeded(self):
        return self.consume_succeeded_at is not None

    def set_consume_started(self, save_immediately=True):
        self.consume_started_at = timezone.now()
        if save_immediately:
            self.save()

    def set_consume_succeeded(self, save_immediately=True):
        self.consume_succeeded_at = timezone.now()
        if save_immediately:
            self.save()

    class Meta:
        abstract = True


class Message(BaseMessage):
    objects = MessageQuerySet.as_manager()

    def move_to_failed_queue(self):
        # In case consumption of the message fails
        # we move the message to a separate failed messages table
        stacktrace_str = "".join(traceback.format_exception(*sys.exc_info()))
        FailedMessage.objects.create(
            created_at=self.created_at,
            collection=self.collection,
            raw_data=self.raw_data,
            consume_started_at=self.consume_started_at,
            consume_succeeded_at=self.consume_succeeded_at,
            consume_failed_at=timezone.now(),
            consume_fail_info=stacktrace_str,
        )

        # remove the original message from the queue
        self.delete()


class FailedMessage(BaseMessage):
    """
    A message for which consumption failed
    """

    consume_failed_at = models.DateTimeField(null=True)
    consume_fail_info = models.TextField(
        null=True, help_text="To store stack traces or other info about the fail"
    )

    objects = FailedMessageQuerySet.as_manager()

    def move_to_ingress_queue(self):
        # In case we want to re-try consumption of the failed message
        # we move the message back to the ingress queue
        Message.objects.create(
            created_at=self.created_at,
            collection=self.collection,
            raw_data=self.raw_data,
            consume_started_at=None,
            consume_succeeded_at=None,
        )

        # remove the failed message from the queue
        self.delete()



from django.core.management.base import BaseCommand

from ingress.models import Collection


class Command(BaseCommand):
    help = "List all existing collections"

    def handle(self, *args, **options):
        collections = Collection.objects.order_by("id")

        self.stdout.write(f"\nCurrent number of collections: {collections.count()}\n")
        if collections.count() == 0:
            return

        table_spacing = "{:<4} {:<25} {:<15} {:<16} {:<10} {:<10} {:<30}"
        header = table_spacing.format(
            "id",
            "name",
            "ingress_enabled",
            "consumer_enabled",
            "unparsed",
            "failed",
            "full url",
        )
        self.stdout.write(header)
        for c in collections:
            self.stdout.write(
                table_spacing.format(
                    c.id,
                    c.name,
                    "yes" if c.ingress_enabled else "no",
                    "yes" if c.consumer_enabled else "no",
                    c.message_set.filter(consume_started_at__isnull=True).count(),
                    c.failedmessage_set.count(),
                    c.url_path,
                )
            )



# Generated by Django 3.0.11 on 2020-11-09 15:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ingress", "0003_failedingressqueue"),
    ]

    operations = [
        migrations.AddField(
            model_name="endpoint",
            name="is_active",
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name="failedingressqueue",
            name="endpoint",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT, to="ingress.Endpoint"
            ),
        ),
    ]



from django.core.management.base import BaseCommand

from ingress.models import Collection


class Command(BaseCommand):
    help = "Deactivate an existing collection"

    def add_arguments(self, parser):
        parser.add_argument(
            "name",
            help="The collection name, used when posting to our api.",
        )

    def handle(self, *args, **options):
        name = options["name"]

        num_rows = Collection.objects.filter(name=name).update(ingress_enabled=False)
        if num_rows > 0:
            self.stdout.write(
                f"Dectivated Collection '{name}', it will now longer accept ingress"
            )
        else:
            self.stderr.write(
                f"Did not deactivate Collection '{name}'. It probably does not exist."
            )



from django.core.management.base import BaseCommand
from django.db import transaction

from ingress.models import Collection, FailedMessage


class Command(BaseCommand):
    help = (
        "Moves all failed messages for the specified collection "
        "back to the main ingress queue to be parsed again."
    )

    def add_arguments(self, parser):
        parser.add_argument(
            "collection_name",
            help=(
                "The name of the collection. Use the 'list_collections' "
                "command to show the existing collections.",
            ),
        )

    def handle(self, *args, **options):
        collection_name = options["collection_name"]

        try:
            collection = Collection.objects.get(name=collection_name)
        except Collection.DoesNotExist:
            self.stderr.write(f"Collection named '{collection_name}' does not exist")
            return

        with transaction.atomic():
            failed_queryset = FailedMessage.objects.for_collection(collection)
            num_failed_messages = failed_queryset.count()
            for failed_message in failed_queryset:
                failed_message.move_to_ingress_queue()

        end_message = (
            f"\n\nMoved {num_failed_messages} messages from the failed queue to "
            f"the normal queue to be parsed again.\n\n"
        )
        self.stdout.write(end_message)



# Generated by Django 2.2.16 on 2020-11-04 16:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ingress", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Endpoint",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("url_key", models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.AlterField(
            model_name="ingressqueue",
            name="endpoint",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="ingress.Endpoint"
            ),
        ),
    ]



import logging

from django.core.management.base import BaseCommand

from ingress.models import Collection


class Command(BaseCommand):
    help = "Disable the consumer for an existing collection"
    logger = logging.getLogger(__name__)

    def add_arguments(self, parser):
        parser.add_argument(
            "name",
            help="The collection name, used when posting to our api.",
        )

    def handle(self, *args, **options):
        name = options["name"]

        num_rows = Collection.objects.filter(name=name).update(consumer_enabled=False)
        if num_rows > 0:
            self.stdout.write(f"Disabled consumer for collection '{name}'.")
        else:
            self.stderr.write(
                f"Did not disable consumer for collection '{name}'. The collection"
                f"probably does not exist."
            )



# Generated by Django 3.0.11 on 2020-11-20 12:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ingress", "0005_auto_20201109_2132"),
    ]

    operations = [
        migrations.AddField(
            model_name="endpoint",
            name="parser_enabled",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.1.5 on 2021-03-15 13:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ingress', '0006_endpoint_parser_enabled'),
    ]

    operations = [
        migrations.RenameField(
            model_name='endpoint',
            old_name='parser_enabled',
            new_name='consumer_enabled',
        ),
        migrations.RenameField(
            model_name='endpoint',
            old_name='is_active',
            new_name='ingress_enabled',
        ),
        migrations.RenameField(
            model_name='failedingressqueue',
            old_name='endpoint',
            new_name='collection',
        ),
        migrations.RenameField(
            model_name='failedingressqueue',
            old_name='parse_failed',
            new_name='parse_failed_at',
        ),
        migrations.RenameField(
            model_name='ingressqueue',
            old_name='endpoint',
            new_name='collection',
        ),
        migrations.RemoveField(
            model_name='ingressqueue',
            name='parse_fail_info',
        ),
        migrations.RemoveField(
            model_name='ingressqueue',
            name='parse_failed',
        ),
        migrations.AlterField(
            model_name='endpoint',
            name='url_key',
            field=models.SlugField(help_text="The end of the url and also the key with which it can be retrieved from the queue. For example, in the url /ingress/example, the string 'example' is the name of the collection.", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name='failedingressqueue',
            name='parse_fail_info',
            field=models.TextField(help_text='To store stack traces or other info about the fail', null=True),
        ),
        migrations.AlterField(
            model_name='failedingressqueue',
            name='parse_started',
            field=models.DateTimeField(db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='failedingressqueue',
            name='parse_succeeded',
            field=models.DateTimeField(db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='failedingressqueue',
            name='raw_data',
            field=models.TextField(help_text='To store raw received data to be consumed by a separate consumer'),
        ),
        migrations.AlterField(
            model_name='ingressqueue',
            name='parse_started',
            field=models.DateTimeField(db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='ingressqueue',
            name='parse_succeeded',
            field=models.DateTimeField(db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='ingressqueue',
            name='raw_data',
            field=models.TextField(help_text='To store raw received data to be consumed by a separate consumer'),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    atomic = False  # Needed because the index is added concurrently

    dependencies = [
        ('ingress', '0009_auto_20210315_1454'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.AlterField(
                    model_name='failedmessage',
                    name='created_at',
                    field=models.DateTimeField(auto_now_add=True, db_index=True),
                )
            ],
            database_operations=[
                migrations.RunSQL(
                    sql="""
                        CREATE INDEX CONCURRENTLY ingress_failedmessage_created_at_5de47ca9
                        ON ingress_failedmessage (created_at);
                    """,
                    reverse_sql="""
                        DROP INDEX ingress_failedmessage_created_at_5de47ca9;
                    """,
                ),
            ],
        ),
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.AlterField(
                    model_name="message",
                    name="created_at",
                    field=models.DateTimeField(auto_now_add=True, db_index=True),
                ),
            ],
            database_operations=[
                migrations.RunSQL(
                    sql="""
                        CREATE INDEX CONCURRENTLY ingress_message_collection_id_c83e46bd
                        ON ingress_message (collection_id);
                    """,
                    reverse_sql="""
                        DROP INDEX ingress_message_collection_id_c83e46bd;
                    """,
                ),
            ],
        ),
    ]



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand

from ingress.models import Collection
from ingress.settings import app_settings


class Command(BaseCommand):
    help = "Consume messages in the queue for a specific collection"
    logger = logging.getLogger(__name__)

    def add_arguments(self, parser):
        parser.add_argument(
            "collection_name",
            help="The collection name for which to consume the messages from the queue",
        )

    def handle(self, *args, **options):
        collection_name = options["collection_name"]

        try:
            collection = Collection.objects.get(name=collection_name)
            consumer_class = app_settings.get_consumer_for_collection(collection.name)
        except Collection.DoesNotExist:
            self.stderr.write(f"The Collection '{collection_name}' does not exist.")
        except ObjectDoesNotExist as e:
            # Consumer does not exist in INGRESS_CONSUMER_CLASSES
            self.stderr.write(str(e))
        else:
            self.stdout.write("Starting consumer 'consumer_class'")
            consumer = consumer_class()
            consumer.consume()
